class BaseHeap(object):
    """Base class of heap queue

    This class should not be instantiated. Subclass need to implement methods
    below.

    A heap is a specialized tree-based data structure that satisfies the heap
    property: If A is a parent node of B then the key (the value) of node A is
    ordered with respect to the key of node B with the same ordering applying
    across the heap. A heap can be classified further as either a "max heap" or
    a "min heap". In a max heap, the keys of parent nodes are always greater
    than or equal to those of the children and the highest key is in the root
    node. In a min heap, the keys of parent nodes are less than or equal to
    those of the children and the lowest key is in the root node. (Wikipedia)

    """

    def __init__(self, **kwargs):
        """Initialization for an xHeap

        Args:
            kwargs (dict): A dictionary contains config info.

        Attributes:
            comp (function): Indicating how to compare two elements.
                Return negtive number if a < b, positive number if a > b,
                0 if they are equal.
        """
        self.comp = lambda a, b: a - b

    def push(self, item):
        """Add a new item to the heap

        Args:
            item (any): The primitive value or object to be added into heap.

        Returns:
            True if successful, False otherwise.
        """
        raise NotImplementedError('subclass with push() implementation '
                                  'is required')

    def pop(self):
        """Delete and return the max/min item.

        Returns:
            The max/min item in the heap.
        """
        raise NotImplementedError('subclass with pop() implementation '
                                  'is required')

    def peek(self):
        """Return the max/min item.

        Return the max/min item in the heap without deleting it.

        Returns:
            The max/min item in the heap.
        """
        raise NotImplementedError('subclass with peek() implementation '
                                  'is required')

    def remove(self, item):
        """Remove item from heap.

        Remove item equals to the given one according to `comp` from the heap.

        Args:
            item (any): The item to be deleted.

        Returns:
            The deleted item.
        """
        raise NotImplementedError('subclass with remove() implementation '
                                  'is required')

    def merge(self, a_heap):
        """Merge two heaps, which have same `comp` result.

        Args:
            a_heap (BaseHeap): The heap to be merged.

        Returns:
            True if successful, False otherwise.
        """
        raise NotImplementedError('subclass with merge() implementation '
                                  'is required')
